#region Copyright Syncfusion Inc. 2001 - 2007
//
//  Copyright Syncfusion Inc. 2001 - 2007. All rights reserved.
//
//  Use of this code is subject to the terms of our license.
//  A copy of the current license can be obtained at any time by e-mailing
//  licensing@syncfusion.com. Any infringement will be prosecuted under
//  applicable laws. 
//
#endregion

using System;
using System.Data;
using System.Drawing;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Runtime.Serialization;
using System.Collections;
using Syncfusion.Windows.Forms.Diagram;

/// <summary>
/// Custom Symbol class capable of containing sub nodes that can be expanded or collapsed.
/// </summary>
[Serializable()]
public class ExpanderSymbol : Syncfusion.Windows.Forms.Diagram.Rectangle
{
    #region Class members
    /// <summary>
    /// Flag indicated for explanded node state.
    /// </summary>
    private bool m_bExpanded = true;
    private Syncfusion.Windows.Forms.Diagram.Label m_lblName = null;
    private Syncfusion.Windows.Forms.Diagram.Label m_lblTitle = null;
    private String m_strManagerID = string.Empty;
    #endregion

    #region Class Initialize/Finalize methods
    /// <summary>
    /// Default constructor for current sample.
    /// </summary>
    public ExpanderSymbol()
        : base(0, 0, 100, 40)
    {
        // Set EditStyle allowing.
        this.EditStyle.AllowChangeWidth = false;
        this.EditStyle.AllowChangeHeight = false;

        this.Name = "Rectangle";
        this.FillStyle.Color = Color.Orange;
        this.FillStyle.ColorAlphaFactor = 200;
        this.LineStyle.LineColor = Color.OrangeRed;

        // enable port
        this.EnableCentralPort = true;

        SizeF szSize = this.BoundsInfo.GetSize(MeasureUnits.Pixel);
        m_lblName = new Syncfusion.Windows.Forms.Diagram.Label(this, "");
        m_lblName.FontStyle.Size = 9;
        m_lblName.OffsetX = szSize.Width / 2;
        m_lblName.OffsetY = szSize.Height * 0.25f;
        m_lblTitle = new Syncfusion.Windows.Forms.Diagram.Label(this, "");
        m_lblTitle.OffsetX = szSize.Width / 2;
        m_lblTitle.OffsetY = szSize.Height * 0.75f;
        this.Labels.Add(m_lblName);
        this.Labels.Add(m_lblTitle);
    }
    /// <summary>
    /// Initializes a new instance of the <see cref="ExpanderSymbol"/> class.
    /// </summary>
    /// <param name="src">The source node.</param>
    public ExpanderSymbol(ExpanderSymbol src)
        : base(src)
    {
        m_bExpanded = src.m_bExpanded;
    }
    /// <summary>
    /// Initializes a new instance of the <see cref="Group"/> class.
    /// </summary>
    /// <param name="info">The serialization Info.</param>
    /// <param name="context">The streaming context.</param>
    /// <summary>
    /// Serialization constructor for symbols.
    /// </summary>
    /// <param name="info">Serialization state information</param>
    /// <param name="context">Streaming context information</param>
    public ExpanderSymbol(SerializationInfo info, StreamingContext context)
        : base(info, context)
    {
        m_bExpanded = info.GetBoolean("isExpanded");
    }
    #endregion

    #region Class properties
    /// <summary>
    /// Get current node state of explanded.
    /// </summary>
    public bool IsExpanded
    {
        get
        {
            return m_bExpanded;
        }
    }
    public string EmployeeName
    {
        get
        {
            return m_lblName.Text;
        }
        set
        {
            m_lblName.Text = value;
        }
    }

    public string EmployeeID
    {
        get
        {
            return m_lblTitle.Text;
        }
        set
        {
            m_lblTitle.Text = value;
        }
    }

    public string ManagerID
    {
        get
        {
            return m_strManagerID;
        }
        set
        {
            m_strManagerID = value;
        }
    }

    // Used by the DiagramLayoutManager to calculate the vertical position of the node in the 
    // employee hierarchy.
    public int Rank
    {
        get
        {
            int rank = 1;
            IGraphNode curNode = this as IGraphNode;
            while (curNode != null)
            {
                ICollection edgesEntering = curNode.EdgesEntering;
                if (edgesEntering.Count > 0)
                {
                    IEnumerator enumEdgesEntering = edgesEntering.GetEnumerator();
                    if (enumEdgesEntering != null)
                    {
                        enumEdgesEntering.MoveNext();
                        IGraphEdge incomingEdge = enumEdgesEntering.Current as IGraphEdge;
                        if (incomingEdge != null)
                        {
                            curNode = incomingEdge.FromNode;
                            rank++;
                        }
                    }
                }
                else
                {
                    curNode = null;
                }
            }

            return rank;
        }
    }

    public NodeCollection SubNodes
    {
        get { return GetSubNodes(); }
    }
    #endregion

    #region Class overrides
    /// <summary>
    /// Clones this instance.
    /// </summary>
    /// <returns></returns>
    public override object Clone()
    {
        return new ExpanderSymbol(this);
    }
    protected override void OnDeserialized()
    {
        base.OnDeserialized();

        // update reference to labels
        if (this.Labels.Count > 0)
        {
            m_lblName = this.Labels[0];
            m_lblTitle = this.Labels[1];
        }
    }
    /// <summary>
    /// Populates a SerializationInfo with the data needed to
    /// serialize the target object.
    /// </summary>
    /// <param name="info">SerializationInfo object to populate</param>
    /// <param name="context">Destination streaming context</param>
    protected override void GetObjectData(SerializationInfo info, StreamingContext context)
    {
        base.GetObjectData(info, context);

        // Add to info new expanded property.
        info.AddValue("isExpanded", m_bExpanded);
    }
    /// <summary>
    /// Renders the specified graphics.
    /// </summary>
    /// <param name="gfx">Graphics to draw on.</param>
    protected override void Render(Graphics gfx)
    {
        base.Render(gfx);

        // draw helper plus/minus symbol
        DrawPlusMinus(gfx);
    }
    #endregion

    #region Class Public Methods
    /// <summary>
    /// Recurse through the child nodes of this parent node and show all child nodes and their connecting links.
    /// </summary>
    /// <param name="clickednode"></param>
    public void ExpandSubNodes(bool clickednode)
    {
        IGraphNode thisNode = this as IGraphNode;

        if (thisNode != null)
        {
            ICollection edgesLeaving = thisNode.EdgesLeaving;
            if (edgesLeaving != null)
            {
                foreach (IGraphEdge curEdge in edgesLeaving)
                {
                    // Get from IGraphEdge link.
                    ConnectorBase outlink = curEdge as ConnectorBase;

                    if (outlink != null)
                    {
                        // Get from link children of current symbol.
                        ExpanderSymbol cursymbol = outlink.ToNode as ExpanderSymbol;
                        if (cursymbol != null)
                        {
                            // Check for explanded and explande nodes in child symbol.
                            if (cursymbol.IsExpanded)
                                cursymbol.ExpandSubNodes(false);

                            // Set current symbol as visible.
                            cursymbol.Visible = true;
                        }

                        // Set connected link as visible too.
                        outlink.Visible = true;
                    }
                }
            }
        }

        // If this symbol is node under mouse click set node state as 
        // expaned by make plus vertical line visibly.
        if (clickednode && HasChild())
        {
            // Set symbol shapes to indicate an expanded state.
            m_bExpanded = true;
        }
    }

    /// <summary>
    /// Recurse through the child nodes of this parent node and hide all child nodes and their connecting links.
    /// </summary>
    /// <param name="clickednode"></param>
    public void CollapseSubNodes(bool clickednode)
    {
        IGraphNode thisNode = this as IGraphNode;
        if (thisNode != null)
        {
            ICollection edgesLeaving = thisNode.EdgesLeaving;
            if (edgesLeaving != null)
            {
                // Convert to collection of IGraphEdges type.
                foreach (IGraphEdge curEdge in edgesLeaving)
                {
                    // Get from IGraphEdge link.
                    ConnectorBase outlink = curEdge as ConnectorBase;
                    if (outlink != null)
                    {
                        // Get from link current symbol children.
                        ExpanderSymbol cursymbol = outlink.ToNode as ExpanderSymbol;
                        if (cursymbol != null)
                        {
                            // Colapse nodes in child symbol.
                            cursymbol.CollapseSubNodes(false);

                            // Set current symbol as invisible.
                            cursymbol.Visible = false;
                        }
                        // Set connected link as invisible too.
                        outlink.Visible = false;
                    }
                }
            }
        }

        // If this symbol is node under mouse click set node state as 
        // expaned by make plus vertical line visibly.
        if (clickednode && HasChild())
        {
            // Set symbol shapes to indicate a collapsed state.
            m_bExpanded = false;
        }
    }
    #endregion

    #region Class helper methods
    /// <summary>
    /// Get Second endpoint inverted to given.
    /// </summary>
    /// <param name="endPoint">The first endPoint</param>
    /// <returns>Inverted  endPoint to given.</returns>
    private EndPoint GetTargetEndPoint(EndPoint endPoint)
    {
        EndPoint endPointToReturn = endPoint;

        IEndPointContainer container = endPoint.Container as IEndPointContainer;

        if (container != null)
        {
            endPointToReturn = (container.HeadEndPoint == endPoint) ? container.TailEndPoint : container.HeadEndPoint;
        }

        return endPoint;
    }
    private bool HasChild()
    {
        return this.EdgesLeaving.Count > 0;
    }
    private void DrawPlusMinus(Graphics gfx)
    {
        if (HasChild())
        {
            int length = 6;
            SizeF szOffset = new SizeF(length, length);

            // horizontal
            PointF[] ptsHorizontal = new PointF[]
					{
						new PointF( szOffset.Width -length/2, szOffset.Height ),
						new PointF( szOffset.Width + length/2, szOffset.Height )
					};

            // vertical
            PointF[] ptsVertical = new PointF[]
					{
						new PointF( szOffset.Width, szOffset.Height-length/2 ),
						new PointF( szOffset.Width, szOffset.Height + length/2 )
					};

            using (Pen penHightlight = new Pen(Color.Gray, 1))
            using (Pen pen = new Pen(Color.Black, 2))
            {
                if (!m_bExpanded)
                {
                    gfx.DrawLine(pen, ptsVertical[0], ptsVertical[1]);
                    gfx.DrawLine(penHightlight, ptsVertical[0], ptsVertical[1]);
                }

                gfx.DrawLine(pen, ptsHorizontal[0], ptsHorizontal[1]);
                gfx.DrawLine(penHightlight, ptsHorizontal[0], ptsHorizontal[1]);
            }
        }
    }
    private NodeCollection GetSubNodes()
    {
        NodeCollection nodes = new NodeCollection();

        foreach (IGraphEdge edge in this.EdgesLeaving)
        {
            Node toNode = edge.ToNode as Node;

            if (toNode != null)
            {
                nodes.Add(toNode);
            }
        }

        return nodes;
    }
    #endregion
}
