#region Copyright Syncfusion Inc. 2001 - 2007
//
//  Copyright Syncfusion Inc. 2001 - 2007. All rights reserved.
//
//  Use of this code is subject to the terms of our license.
//  A copy of the current license can be obtained at any time by e-mailing
//  licensing@syncfusion.com. Any infringement will be prosecuted under
//  applicable laws. 
//
#endregion

using System;
using System.Data;
using System.Drawing;
using System.Xml;
using System.Configuration;
using System.Web;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;
using System.Collections;
using System.Diagnostics;

using Syncfusion.Windows.Forms.Diagram;
using Syncfusion.Web.UI.WebControls.Diagram;

public partial class Expander : System.Web.UI.Page 
{
    protected DirectedTreeLayoutManager dtLayoutMgr;

    protected void Page_Load(object sender, EventArgs e)
    {
        if (this.IsPostBack == false)
        {
            this.dtLayoutMgr = new DirectedTreeLayoutManager();
            this.dtLayoutMgr.VerticalSpacing = 50;
            this.dtLayoutMgr.HorizontalSpacing = 25;
            this.DiagramWebControl1.LayoutManager = this.dtLayoutMgr;

            this.DiagramWebControl1.Model.BackgroundStyle.Color = Color.FromArgb(246, 249, 255);
            this.DiagramWebControl1.Model.LineStyle.LineColor = Color.SteelBlue;
            // Initialize the DiagramWebControl with employee data from an XML data file
            this.InitializeDiagramFromXMLData();
            
           
            // Add a TextNode to highlight the subnode feature
            string strText = "Double-click to expand/collapse sub nodes";
            Syncfusion.Windows.Forms.Diagram.TextNode txtnode = new TextNode(strText);
            txtnode.FontStyle.Size = 14;
            txtnode.FontStyle.Bold = true;
            txtnode.FontStyle.Family = "Arial";
            txtnode.FontColorStyle.Color = Color.Green ;
            txtnode.LineStyle.LineColor = Color.LightBlue;
            txtnode.SizeToText(new SizeF(500, 50));
            txtnode.EditStyle.AllowSelect = false;

            ((IUnitIndependent)txtnode).SetPinPoint(new PointF(220, 30), MeasureUnits.Pixel);
            this.DiagramWebControl1.Model.AppendChild(txtnode);

            this.DiagramWebControl1.Model.SizeToContent = true;
            this.DiagramWebControl1.Model.RenderingStyle.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;
            this.DiagramWebControl1.Model.RenderingStyle.TextRenderingHint = System.Drawing.Text.TextRenderingHint.ClearTypeGridFit;

        }
        this.DiagramWebControl1.RenderImageMap = true;

        this.DiagramWebControl1.PostbackDescriptors += new PostbackDescriptorsEventHandler(this.DiagramWebControl1_PostbackDescriptors);
        //this.DiagramWebControl1.NodeClick += new NodeClickEventHandler(DiagramWebControl1_NodeClick);
        this.DiagramWebControl1.NodeDoubleClick += new NodeDoubleClickEventHandler(this.Diagram1_NodeDoubleClick);
    }

    protected void Page_UnLoad(object sender, System.EventArgs e)
    {
        this.DiagramWebControl1.PostbackDescriptors -= new PostbackDescriptorsEventHandler(this.DiagramWebControl1_PostbackDescriptors);
        //this.DiagramWebControl1.NodeClick += new NodeClickEventHandler(DiagramWebControl1_NodeClick);
        this.DiagramWebControl1.NodeDoubleClick -= new NodeDoubleClickEventHandler(this.Diagram1_NodeDoubleClick);
    }

    #region Diagram Initialization

    // Gets the employee data from an XML format file and creates the diagram organization tree.
    private void InitializeDiagramFromXMLData()
    {
        string datasrcpath = Server.MapPath(string.Empty);
        datasrcpath = datasrcpath + @"\App_Data\Employees.xml";
        // Temporarily suspend the layoutmanager
        this.DiagramWebControl1.LayoutManager.AutoLayout = false;
        this.DiagramWebControl1.Model.BeginUpdate();

        // Read data from the XML file and populate a Hashtable using the employee ID as the key
        Hashtable htemployees = this.ReadEmployeeDataFromXMLFile(datasrcpath);

        foreach (Node node in htemployees.Values)
        {
            this.DiagramWebControl1.Model.AppendChild(node);
        }

        foreach (ExpanderSymbol emply in htemployees.Values)
        {
            // get parent
            ExpanderSymbol manager = (ExpanderSymbol)htemployees[emply.ManagerID];

            ConnectNodes(manager, emply);
        }

        this.DiagramWebControl1.Model.EndUpdate();

        // Instruct the LayoutManager to calculate and layout the DiagramWebControl1 nodes
        this.DiagramWebControl1.LayoutManager.Nodes.AddRange(this.DiagramWebControl1.Model.Nodes);
        this.DiagramWebControl1.LayoutManager.UpdateLayout(null);

        htemployees.Clear();
    }

    private void ConnectNodes(ExpanderSymbol parent, ExpanderSymbol child)
    {
        if (parent != null && child != null && parent.EnableCentralPort && child.EnableCentralPort)
        {
            // Custom orthogonal link showing the connection between parent nodes and sub nodes
            LineConnector link = new LineConnector(PointF.Empty, new PointF(10, 10));
            link.LineStyle.LineColor = Color.SteelBlue;
            link.HeadDecorator.DecoratorShape = DecoratorShape.FilledFancyArrow;
            link.HeadDecorator.Size = new SizeF(12, 9);
            link.HeadDecorator.FillStyle.Color = Color.Navy;
            link.HeadDecorator.LineStyle.LineColor = Color.Transparent;
            this.DiagramWebControl1.Model.AppendChild(link);

            parent.CentralPort.TryConnect(link.TailEndPoint);
            child.CentralPort.TryConnect(link.HeadEndPoint);

            this.DiagramWebControl1.Model.SendToBack(link);
        }
    }

    // Use the XMLDocument class to read the org chart data from the exported XML database file
    private Hashtable ReadEmployeeDataFromXMLFile(string filepath)
    {
        Hashtable htemployees = new Hashtable();

        // Use the XML DOM to read data from the XML org chart data file
        try
        {
            XmlDocument xmldoc = new XmlDocument();
            xmldoc.Load(filepath);
            if (xmldoc.DocumentElement.HasChildNodes)
            {
                XmlNodeList employeelist = xmldoc.DocumentElement.ChildNodes;
                for (int i = 0; i < employeelist.Count; i++)
                {
                    // Create an ExpanderSymbol instance to represent each employee
                    ExpanderSymbol expsymbol = new ExpanderSymbol();

                    XmlNode employeenode = employeelist[i];
                    XmlNodeList employeedatalist = employeenode.ChildNodes;
                    IEnumerator employeedata = employeedatalist.GetEnumerator();
                    employeedata.Reset();
                    while (employeedata.MoveNext())
                    {
                        XmlNode dataelement = employeedata.Current as XmlNode;
                        switch (dataelement.Name)
                        {
                            case "Name":
                                expsymbol.EmployeeName = dataelement.InnerText;
                                break;
                            case "EmployeeID":
                                expsymbol.EmployeeID = dataelement.InnerText;
                                break;
                            case "ManagerID":
                                expsymbol.ManagerID = dataelement.InnerText;
                                break;
                        }
                    }
                    htemployees.Add(expsymbol.EmployeeID, expsymbol);
                }
            }
        }
        catch
        {
            Debug.Assert(false,"Failed to load employee data.", "Essential Diagram Sample");
        }

        return htemployees;
    }

   #endregion

    #region DiagramWebControl EventHandlers

    // Handler for the DiagramWebControl.PostbackDescriptors event.
    // This event allows the application to add the required client-side script for posting user 
    // events on the diagram image elements back to the diagram server control.
    private void DiagramWebControl1_PostbackDescriptors(object sender, EventArgs evtargs)
    {
        this.DiagramWebControl1.PostbackEventDescriptors.Add(PostbackEvents.NodeDoubleClickMarker, new NodeDoubleClickEventDescriptor(this.Request.Browser));
        //this.DiagramWebControl1.PostbackEventDescriptors.Add(PostbackEvents.NodeClickMarker, new NodeClickEventDescriptor(this.Request.Browser));
    }

    // Handler for the DiagramWebControl.NodeClick event. 
    // Clicking the ExpanderSymbol node will show/hide the child nodes belonging to the symbol.
    private void DiagramWebControl1_NodeClick(object sender, NodeClickEventArgs e)
    {
        if (e.Node is ExpanderSymbol)
        {
            ExpanderSymbol expsymbol = e.Node as ExpanderSymbol;
            if (expsymbol.SubNodes.Count > 0)
            {
                this.DiagramWebControl1.Model.BeginUpdate();
                if (expsymbol.IsExpanded == false)
                {
                    expsymbol.ExpandSubNodes(true);
                }
                else
                {
                    expsymbol.CollapseSubNodes(true);
                }
                this.DiagramWebControl1.Model.EndUpdate();
            }
        }
    }

    // Handler for the DiagramWebControl.NodeDoubleClick event. 
    // DoubleClicking the ExpanderSymbol node will show/hide the child nodes belonging to the symbol.
    private void Diagram1_NodeDoubleClick(object sender, NodeDoubleClickEventArgs e)
    {
        if (e.Node is ExpanderSymbol)
        {
            ExpanderSymbol expsymbol = (ExpanderSymbol)e.Node;
            if (expsymbol.SubNodes.Count > 0)
            {
                this.DiagramWebControl1.Model.BeginUpdate();
                if (expsymbol.IsExpanded == false)
                {
                    expsymbol.ExpandSubNodes(true);
                }
                else
                {
                    expsymbol.CollapseSubNodes(true);
                }
                this.DiagramWebControl1.Model.EndUpdate();
            }
        }
    }
    #endregion		
}